@@ -142,8 +142,9 @@ densityplot <-
              to = NULL,
              cut = NULL,
              na.rm = NULL,
-             drop.unused.levels = TRUE,
+             drop.unused.levels = lattice.getOption("drop.unused.levels"),
              ...,
+             default.scales = list(),
              panel.groups = "panel.densityplot",
              subscripts = !is.null(groups),
              subset = TRUE)
@@ -232,6 +233,7 @@ densityplot <-
 
     ## scales <- eval(substitute(scales), data, parent.frame())
     if (is.character(scales)) scales <- list(relation = scales)
+    scales <- updateList(default.scales, scales)
     foo <- c(foo,
              do.call("construct.scales", scales))
 
