@@ -168,8 +168,9 @@ xyplot <-
              xlim,
              ylab,
              ylim,
-             drop.unused.levels = TRUE,
+             drop.unused.levels = lattice.getOption("drop.unused.levels"),
              ...,
+             default.scales = list(),
              subscripts = !is.null(groups),
              subset = TRUE)
 {
@@ -245,6 +246,7 @@ xyplot <-
 
     ##scales <- eval(substitute(scales), data, parent.frame())
     if (is.character(scales)) scales <- list(relation = scales)
+    scales <- updateList(default.scales, scales)
     foo <- c(foo, 
              do.call("construct.scales", scales))
 
